Class: Gitlab::Git::Repository

Inherits:
Object
  • Object
show all
Includes:
Popen
Defined in:
lib/gitlab_git/repository.rb

Defined Under Namespace

Classes: NoRepository

Instance Attribute Summary collapse

Instance Method Summary collapse

Methods included from Popen

#popen

Constructor Details

#initialize(path) ⇒ Repository

Returns a new instance of Repository.



28
29
30
31
32
# File 'lib/gitlab_git/repository.rb', line 28

def initialize(path)
  @path = path
  @name = path.split("/").last
  @root_ref = discover_default_branch
end

Instance Attribute Details

#grit ⇒ Object (readonly)

Grit repo object



23
24
25
# File 'lib/gitlab_git/repository.rb', line 23

def grit
  @grit
end

#name ⇒ Object (readonly)

Directory name of repo



20
21
22
# File 'lib/gitlab_git/repository.rb', line 20

def name
  @name
end

#path ⇒ Object (readonly)

Full path to repo



17
18
19
# File 'lib/gitlab_git/repository.rb', line 17

def path
  @path
end

#root_ref ⇒ Object

Default branch in the repository



14
15
16
# File 'lib/gitlab_git/repository.rb', line 14

def root_ref
  @root_ref
end

#rugged ⇒ Object (readonly)

Rugged repo object



26
27
28
# File 'lib/gitlab_git/repository.rb', line 26

def rugged
  @rugged
end

Instance Method Details

#archive_repo(ref, storage_path, format = "tar.gz") ⇒ Object

Archive Project to .tar.gz

Already packed repo archives stored at app_root/tmp/repositories/project_name/project_name-commit-id.tag.gz



126
127
128
129
130
131
132
133
134
135
136
137
138
139
140
141
142
143
144
145
146
147
148
149
150
151
152
153
154
155
156
157
158
159
160
161
162
163
164
165
166
167
# File 'lib/gitlab_git/repository.rb', line 126

def archive_repo(ref, storage_path, format = "tar.gz")
  ref = ref || self.root_ref
  commit = Gitlab::Git::Commit.find(self, ref)
  return nil unless commit

  extension = nil
  git_archive_format = nil
  pipe_cmd = nil

  case format
  when "tar.bz2", "tbz", "tbz2", "tb2", "bz2"
    extension = ".tar.bz2"
    pipe_cmd = "bzip2"
  when "tar"
    extension = ".tar"
    pipe_cmd = "cat"
  when "zip"
    extension = ".zip"
    git_archive_format = "zip"
    pipe_cmd = "cat"
  else
    # everything else should fall back to tar.gz
    extension = ".tar.gz"
    git_archive_format = nil
    pipe_cmd = "gzip"
  end

  # Build file path
  file_name = self.name.gsub("\.git", "") + "-" + commit.id.to_s + extension
  file_path = File.join(storage_path, self.name, file_name)

  # Put files into a directory before archiving
  prefix = File.basename(self.name) + "/"

  # Create file if not exists
  unless File.exists?(file_path)
    FileUtils.mkdir_p File.dirname(file_path)
    file = self.grit.archive_to_file(ref, prefix, file_path, git_archive_format, pipe_cmd)
  end

  file_path
end

#branch_names ⇒ Object

Returns an Array of branch names sorted by name ASC



53
54
55
# File 'lib/gitlab_git/repository.rb', line 53

def branch_names
  branches.map(&:name)
end

#branch_names_contains(commit) ⇒ Object

Returns branch names collection that contains the special commit(SHA1 or name)

Ex. repo.branch_names_contains('master')



298
299
300
301
302
303
304
305
306
307
308
309
# File 'lib/gitlab_git/repository.rb', line 298

def branch_names_contains(commit)
  output = grit.git.native(:branch, {contains: true}, commit)

  # Fix encoding issue
  output = EncodingHelper::encode!(output)

  # The output is expected as follow
  #   fix-aaa
  #   fix-bbb
  # * master
  output.scan(/[^* \n]+/)
end

#branches ⇒ Object

Returns an Array of Branches



58
59
60
61
62
63
64
# File 'lib/gitlab_git/repository.rb', line 58

def branches
  rugged.refs.select do |ref|
    ref.name =~ /\Arefs\/heads/
  end.map do |rugged_ref|
    Branch.new(rugged_ref.name, rugged_ref.target)
  end.sort_by(&:name)
end

#commits_between(from, to) ⇒ Object

Delegate commits_between to Grit method



219
220
221
# File 'lib/gitlab_git/repository.rb', line 219

def commits_between(from, to)
  grit.commits_between(from, to)
end

#diff(from, to, *paths) ⇒ Object



227
228
229
# File 'lib/gitlab_git/repository.rb', line 227

def diff(from, to, *paths)
  grit.diff(from, to, *paths)
end

#discover_default_branch ⇒ Object

Discovers the default branch based on the repository's available branches

  • If no branches are present, returns nil
  • If one branch is present, returns its name
  • If two or more branches are present, returns current HEAD or master or first branch


107
108
109
110
111
112
113
114
115
116
117
118
119
# File 'lib/gitlab_git/repository.rb', line 107

def discover_default_branch
  if branch_names.length == 0
    nil
  elsif branch_names.length == 1
    branch_names.first
  elsif rugged.head && branch_names.include?(Ref.extract_branch_name(rugged.head.name))
    Ref.extract_branch_name(rugged.head.name)
  elsif branch_names.include?("master")
    "master"
  else
    branch_names.first
  end
end

#empty? ⇒ Boolean

Returns:

  • (Boolean)


94
95
96
# File 'lib/gitlab_git/repository.rb', line 94

def empty?
  rugged.empty?
end

#find_commits(options = {}) ⇒ Object

Returns commits collection

Ex. repo.find_commits( ref: 'master', max_count: 10, skip: 5, order: :date )

+options+ is a Hash of optional arguments to git
:ref is the ref from which to begin (SHA1 or name)
:contains is the commit contained by the refs from which to begin (SHA1 or name)
:max_count is the maximum number of commits to fetch
:skip is the number of commits to skip
:order is the commits order and allowed value is :date(default) or :topo


248
249
250
251
252
253
254
255
256
257
258
259
260
261
262
263
264
265
266
267
268
269
270
271
272
273
274
275
276
277
278
279
280
281
282
283
284
285
286
287
288
289
290
291
# File 'lib/gitlab_git/repository.rb', line 248

def find_commits(options = {})
  actual_options = options.dup

  allowed_options = [:ref, :max_count, :skip, :contains, :order]

  actual_options.keep_if do |key, value|
    allowed_options.include?(key)
  end

  default_options = {pretty: 'raw', order: :date}

  actual_options = default_options.merge(actual_options)

  order = actual_options.delete(:order)

  case order
  when :date
    actual_options[:date_order] = true
  when :topo
    actual_options[:topo_order] = true
  end

  ref = actual_options.delete(:ref)

  containing_commit = actual_options.delete(:contains)

  args = []

  if ref
    args.push(ref)
  elsif containing_commit
    args.push(*branch_names_contains(containing_commit))
  else
    actual_options[:all] = true
  end

  output = grit.git.native(:rev_list, actual_options, *args)

  Grit::Commit.list_from_string(grit, output).map do |commit|
    Gitlab::Git::Commit.decorate(commit)
  end
rescue Grit::GitRuby::Repository::NoSuchShaFound
  []
end

#has_commits? ⇒ Boolean

Returns:

  • (Boolean)


90
91
92
# File 'lib/gitlab_git/repository.rb', line 90

def has_commits?
  !empty?
end

#heads ⇒ Object

Deprecated. Will be removed in 5.2



86
87
88
# File 'lib/gitlab_git/repository.rb', line 86

def heads
  @heads ||= grit.heads.sort_by(&:name)
end

#log(options) ⇒ Object

Delegate log to Grit method

Usage. repo.log( ref: 'master', path: 'app/models', limit: 10, offset: 5, )



197
198
199
200
201
202
203
204
205
206
207
208
209
210
211
212
213
214
215
# File 'lib/gitlab_git/repository.rb', line 197

def log(options)
  default_options = {
    limit: 10,
    offset: 0,
    path: nil,
    ref: root_ref,
    follow: false
  }

  options = default_options.merge(options)

  grit.log(
    options[:ref] || root_ref,
    options[:path],
    max_count: options[:limit].to_i,
    skip: options[:offset].to_i,
    follow: options[:follow]
  )
end

#lookup(oid) ⇒ Object

Lookup for rugged object by oid



326
327
328
# File 'lib/gitlab_git/repository.rb', line 326

def lookup(oid)
  rugged.lookup(oid)
end

#merge_base_commit(from, to) ⇒ Object



223
224
225
# File 'lib/gitlab_git/repository.rb', line 223

def merge_base_commit(from, to)
  grit.git.native(:merge_base, {}, [to, from]).strip
end

#raw ⇒ Object

Alias to old method for compatibility



41
42
43
# File 'lib/gitlab_git/repository.rb', line 41

def raw
  grit
end

#ref_names ⇒ Object

Returns an Array of branch and tag names



81
82
83
# File 'lib/gitlab_git/repository.rb', line 81

def ref_names
  branch_names + tag_names
end

#refs_hash ⇒ Object

Get refs hash which key is SHA1 and value is ref object(Grit::Head or Grit::Remote or Grit::Tag)



313
314
315
316
317
318
319
320
321
322
323
# File 'lib/gitlab_git/repository.rb', line 313

def refs_hash
  # Initialize only when first call
  if @refs_hash.nil?
    @refs_hash = Hash.new { |h, k| h[k] = [] }

    grit.refs.each do |r|
      @refs_hash[r.commit.id] << r
    end
  end
  @refs_hash
end

#repo_exists? ⇒ Boolean

Returns:

  • (Boolean)


98
99
100
# File 'lib/gitlab_git/repository.rb', line 98

def repo_exists?
  !!rugged
end

#search_files(query, ref = nil) ⇒ Object



175
176
177
178
179
180
181
182
183
184
185
# File 'lib/gitlab_git/repository.rb', line 175

def search_files(query, ref = nil)
  if ref.nil? || ref == ""
    ref = root_ref
  end

  greps = grit.grep(query, 3, ref)

  greps.map do |grep|
    Gitlab::Git::BlobSnippet.new(ref, grep.content, grep.startline, grep.filename)
  end
end

#size ⇒ Object

Return repo size in megabytes



170
171
172
173
# File 'lib/gitlab_git/repository.rb', line 170

def size
  size = popen('du -s', path).first.strip.to_i
  (size.to_f / 1024).round(2)
end

#submodules(ref) ⇒ Object

Return hash with submodules info for this repository

Ex. { "rack" => { "id" => "c67be4624545b4263184c4a0e8f887efd0a66320", "path" => "rack", "url" => "git://github.com/chneukirchen/rack.git" }, "encoding" => { "id" => .... } }



344
345
346
# File 'lib/gitlab_git/repository.rb', line 344

def submodules(ref)
  Grit::Submodule.config(grit, ref)
end

#tag_names ⇒ Object

Returns an Array of tag names



67
68
69
# File 'lib/gitlab_git/repository.rb', line 67

def tag_names
  tags.map(&:name)
end

#tags ⇒ Object

Returns an Array of Tags



72
73
74
75
76
77
78
# File 'lib/gitlab_git/repository.rb', line 72

def tags
  rugged.refs.select do |ref|
    ref.name =~ /\Arefs\/tags/
  end.map do |rugged_ref|
    Tag.new(rugged_ref.name, rugged_ref.target)
  end.sort_by(&:name)
end