Class: Gitlab::Git::DiffCollection
- Inherits:
-
Object
- Object
- Gitlab::Git::DiffCollection
- Includes:
- Enumerable
- Defined in:
- lib/gitlab_git/diff_collection.rb
Constant Summary collapse
- DEFAULT_LIMITS =
{ max_files: 100, max_lines: 5000 }.freeze
Instance Method Summary collapse
- #decorate! ⇒ Object
- #each ⇒ Object
- #empty? ⇒ Boolean
-
#initialize(iterator, options = {}) ⇒ DiffCollection
constructor
A new instance of DiffCollection.
- #overflow? ⇒ Boolean
- #real_size ⇒ Object
- #size ⇒ Object
Constructor Details
#initialize(iterator, options = {}) ⇒ DiffCollection
Returns a new instance of DiffCollection.
8 9 10 11 12 13 14 15 16 17 18 19 20 21 22 23 |
# File 'lib/gitlab_git/diff_collection.rb', line 8 def initialize(iterator, ={}) @iterator = iterator @max_files = .fetch(:max_files, DEFAULT_LIMITS[:max_files]) @max_lines = .fetch(:max_lines, DEFAULT_LIMITS[:max_lines]) @max_bytes = @max_files * 5120 # Average 5 KB per file @safe_max_files = [@max_files, DEFAULT_LIMITS[:max_files]].min @safe_max_lines = [@max_lines, DEFAULT_LIMITS[:max_lines]].min @safe_max_bytes = @safe_max_files * 5120 # Average 5 KB per file @all_diffs = !!.fetch(:all_diffs, false) @no_collapse = !!.fetch(:no_collapse, true) @line_count = 0 @byte_count = 0 @overflow = false @array = Array.new end |
Instance Method Details
#decorate! ⇒ Object
97 98 99 100 101 |
# File 'lib/gitlab_git/diff_collection.rb', line 97 def decorate! each_with_index do |element, i| @array[i] = yield(element) end end |
#each ⇒ Object
25 26 27 28 29 30 31 32 33 34 35 36 37 38 39 40 41 42 43 44 45 46 47 48 49 50 51 52 53 54 55 56 57 58 59 60 61 62 63 64 65 66 67 68 69 70 71 72 |
# File 'lib/gitlab_git/diff_collection.rb', line 25 def each @iterator.each_with_index do |raw, i| # First yield cached Diff instances from @array if @array[i] yield @array[i] next end # We have exhausted @array, time to create new Diff instances or stop. break if @overflow if !@all_diffs && i >= @max_files @overflow = true break end # Going by the number of files alone it is OK to create a new Diff instance. diff = Gitlab::Git::Diff.new(raw) # If a diff is too large we still want to display some information # about it (e.g. the file path) without keeping the raw data around # (as this would be a waste of memory usage). # # This also removes the line count (from the diff itself) so it # doesn't add up to the total amount of lines. if diff.too_large? diff.prune_large_diff! end if !@all_diffs && !@no_collapse if diff.collapsible? || over_safe_limits?(i) diff.prune_collapsed_diff! end end @line_count += diff.line_count @byte_count += diff.diff.bytesize if !@all_diffs && (@line_count >= @max_lines || @byte_count >= @max_bytes) # This last Diff instance pushes us over the lines limit. We stop and # discard it. @overflow = true break end yield @array[i] = diff end end |
#empty? ⇒ Boolean
74 75 76 |
# File 'lib/gitlab_git/diff_collection.rb', line 74 def empty? !@iterator.any? end |
#overflow? ⇒ Boolean
78 79 80 81 |
# File 'lib/gitlab_git/diff_collection.rb', line 78 def overflow? populate! !!@overflow end |
#real_size ⇒ Object
87 88 89 90 91 92 93 94 95 |
# File 'lib/gitlab_git/diff_collection.rb', line 87 def real_size populate! if @overflow "#{size}+" else size.to_s end end |
#size ⇒ Object
83 84 85 |
# File 'lib/gitlab_git/diff_collection.rb', line 83 def size @size ||= count # forces a loop through @iterator end |