Method: Gitlab::Git::Repository#diff_text
- Defined in:
- lib/gitlab_git/repository.rb
#diff_text(from, to, options = {}, *paths) ⇒ Object
Return the diff between from and to in a single patch string. The options hash has the same allowed keys as #diff.
284 285 286 287 288 289 290 291 |
# File 'lib/gitlab_git/repository.rb', line 284 def diff_text(from, to, = {}, *paths) # NOTE: It would be simpler to use the Rugged::Diff#patch method, but # that formats the diff text differently than Rugged::Patch#to_s for # changes to binary files. diff_patches(from, to, , *paths).map do |p| p.to_s end.join("\n") end |